from typing import Any, Dict, List

from paddlex_hps_server import BaseTritonPythonModel, schemas, utils


class TritonPythonModel(BaseTritonPythonModel):
    def get_input_model_type(self):
        return schemas.open_vocabulary_detection.InferRequest

    def get_result_model_type(self):
        return schemas.open_vocabulary_detection.InferResult

    def run(self, input, log_id):
        file_bytes = utils.get_raw_bytes(input.image)
        image = utils.image_bytes_to_array(file_bytes)

        result = list(
            self.pipeline.predict(
                image,
                prompt=input.prompt,
                thresholds=input.thresholds,
            )
        )[0]

        objects: List[Dict[str, Any]] = []
        for obj in result["boxes"]:
            objects.append(
                dict(
                    bbox=obj["coordinate"],
                    categoryName=obj["label"],
                    score=obj["score"],
                )
            )
        if self.app_config.visualize:
            output_image_base64 = utils.base64_encode(
                utils.image_to_bytes(result.img["res"])
            )
        else:
            output_image_base64 = None

        return schemas.open_vocabulary_detection.InferResult(
            detectedObjects=objects, image=output_image_base64
        )
